/**********************************************************************

This file is part of the Quantum Computation Language QCL.

(c) Copyright by Bernhard Oemer <oemer@tph.tuwien.ac.at>, 1998

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

     This program is free software under the terms of the 
     GNU General Public Licence (GPL) version 2 or higher

************************************************************************/


#ifndef ERROR_H
#define ERROR_H 1

#pragma interface

#include "types.h"

int yyerror(char*);

enum ErrType { errNONE, errEXIT, errMSG, errINT, errLEXICAL, errPARSE, 
               errTYPMIS, errINVTYP, errPARMIS, errINVPAR, errSYMBOL, 
	       errSCOPE, errDEF, errMATH,errRUN, errGEN, errSYNTAX,
	       errUSR, errRANGE, errEXT, errIO, errSHELL, errMEM,
	       errOPT };



void initialize_readline();
void qclerror(string e,string m="");
void qclabort(string e="",string m="");
string qclinput(string p="?");
void qcloutput(string s);

extern int isErrorReported;

class tError {
  ErrType err;
  int lineno;
  string msg;
  string linetxt;
  tId sym;
  ObjType obj;
  int traceflag;
  sObject *object;
  SymTable *local;
  SymTable *global;
  QuHeap *quheap;
  sDef *def;

  ErrType errType;
  string errMsg;
  sDef *errDef;
  sObject *errStmt;
  sObject *errObj;  
  
  int trTypeLevel;
  int trExecLevel;
public:

  int level;
  void reset();
  tError() { reset(); traceflag=0; level=0; }
  void printerr(ostream& o);
  string errmsg();
  int isMsg() { return err==errMSG; }
  int isErr() { return err>errMSG; }
  int isNone() { return err==errNONE; }
  int isTrace() { return traceflag; }
  int isExit() { return err==errEXIT; }
  ErrType errtype() { return err; }
  void report(ErrType e) { err=e; msg=""; }
  void report(ErrType e,sDef *d);
  void report(ErrType e,int l) { err=e; lineno=l; msg=""; }
  void remark(string s) { msg+=s; }
  void line(string txt,int l);
  void settrace(int t) { traceflag=t; }
  void print(string s);
  void log(string s);
  string input(string s);
  void trace(sObject *obj,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *d);
    
  void trBeginTypecheck(sObject *obj,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *d);
  void trEndTypecheck(sObject *obj,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *d);
  void trBeginExec(sObject *obj,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *d);
  void trEndExec(sObject *obj,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *d);
 
};

#endif
