/**********************************************************************

This file is part of the Quantum Computation Language QCL.

(c) Copyright by Bernhard Oemer <oemer@tph.tuwien.ac.at>, 1998

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

     This program is free software under the terms of the 
     GNU General Public Licence (GPL) version 2 or higher

************************************************************************/


#include "syntax.h"
#include "symbols.h"
#include "error.h"
#include "parse.h"
#include "options.h"

sObject *yyObject;
int yyTest;
FILE* yyToplevelFile;
int yyStringBufferActive;
string yyFilenames[YYMAXINCLUDE];
FILE* yyFilePointers[YYMAXINCLUDE];
int yyLineNos[YYMAXINCLUDE];
int yyIncludeStackPt = 0;
string yyFilename="<input>";

extern char *yytext;
extern int yylineno;

int yyerror(char* s) {
  string t="syntax error: ";
  if(!yyStringBufferActive) {
    t+="in "+yyFilename+", line "+dec(yylineno)+", ";
  };
  t+="near \""+string(yytext)+"\"";
  qclerror(t+": "+s);
  return 1;
}

FILE* openqclfile(string fname) {
  string s;
  FILE *f;
  s=fname;
  if(f=fopen(s,"r")) { yyFilename=s; return f; };
  s=fname+".qcl";
  if(f=fopen(s,"r")) { yyFilename=s; return f; };
  s=optIncludePath+"/"+fname;
  if(f=fopen(s,"r")) { yyFilename=s; return f; };
  s=optIncludePath+"/"+fname+".qcl";
  if(f=fopen(s,"r")) { yyFilename=s; return f; };
  return 0;
}

int do_chksyntax() {
  int r;
  while(1) {
    yyObject=0;
    r=yyparse();
    if(r) {
      if(yyObject && yyObject->object()==sMSG)
    	qclerror(((sMsg*)yyObject)->msg());
      if(yyObject) qcl_delete(yyObject);
      return 1;
    }
//  if(!yyObject) qclabort("syntax check failed");
    if(yyObject && yyObject->isEof()) {
      qcl_delete(yyObject);
      return 0;
    }
    if(yyObject) qcl_delete(yyObject);
    yyObject=0;
  }
}

int chksyntax(string s) {
  int t;
  int r;
  yyTest=1;
  yyObject=0;
  yyScanString(s);
  r=do_chksyntax();
  yyCleanUp();
  yyTest=0;
  return r;
}

int chksyntax(FILE* f) {
  int t;
  int r;
  yyTest=1;
  yyObject=0;
  yyScanFile(f);
  r=do_chksyntax();
  yyCleanUp();
  yyTest=0;
  return r;
}

sObject* parseobj() {
  yyTest=0;
  yyObject=0;
  int r;
  r=yyparse();
  if(r) {
    if(yyObject && yyObject->object()==sMSG)
      qclabort(((sMsg*)yyObject)->msg());
  };
  if(r || !yyObject) qclabort("parsing failed");
  if(yyObject->isEof()) {
    qcl_delete(yyObject);
    return 0;
  }
  return yyObject;
}
