/**********************************************************************

This file is part of the Quantum Computation Language QCL.

(c) Copyright by Bernhard Oemer <oemer@tph.tuwien.ac.at>, 1998

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

     This program is free software under the terms of the 
     GNU General Public Licence (GPL) version 2 or higher

************************************************************************/


#ifndef QUHEAP_H
#define QUHEAP_H

#pragma interface

#include <DLList.h>
#include <fstream.h>

#include "operator.h"
#include "types.h"
#include "syntax.h"

extern int isStateModified;

term string2term(const string& s,int l);
string term2string(const term& t,int binary=0);

class QuHeap {
  struct quentry {
    sRoutDef *d;
    SymTable *s;
    int i;
  };
  typedef DLList<quentry> qulist;
  int nbits;
  bitvec mbits;
  int nfree;
  bitvec mfree;
  int ndirty;
  bitvec mdirty;
  quState *bs;
  QuHeap *parent;
  int delbs;
  int delay;
  qulist *ql;
public:
  QuHeap(int n);
  QuHeap(quState *s,int d=0);
  QuHeap(QuHeap *h,int d=0);
  ~QuHeap();
  tValue qualloc(int n,BaseType t=tQUREG); 
  int qufree(tValue var);
  tValue measure(tValue);
  int reset();
  int add(sRoutDef *d,SymTable *s,int inv);
  int apply(SymTable *gl,tError *err,int inv);
  int addFanout(SymTable *gl,tError *err,tValue a,tValue b,int inv);
  int applyFanout(SymTable *gl,tError *err,tValue a,tValue b,int inv);
  int addSwap(SymTable *gl,tError *err,tValue a,tValue b,int inv);
  int applySwap(SymTable *gl,tError *err,tValue a,tValue b,int inv);
  string prtstr();

  quState *state() { return bs; }
  int nBits() { return nbits; }
  bitvec mBits() { return mbits; }
  int nFree() { return nfree; }
  bitvec mFree() { return mfree; }
  int nDirty() { return ndirty; }
  bitvec mDirty() { return mdirty; }
  int isDelayed() { return delay; }
  
  int load(ifstream& f);
  int save(ofstream& f);
};

#endif
