/**********************************************************************

This file is part of the Quantum Computation Language QCL.

(c) Copyright by Bernhard Oemer <oemer@tph.tuwien.ac.at>, 1998

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

     This program is free software under the terms of the 
     GNU General Public Licence (GPL) version 2 or higher

************************************************************************/


#pragma implementation

#include "types.h"

const char *FALSE_STR="false";
const char *TRUE_STR="true";

const char *BASETYPE_STR[] = {"%UNDEF%", "%ERROR%", "procedure", "operator", 
  "qufunct", "boolean", "int", "real", "complex", "string", "quconst",
  "qureg", "quvoid", "quscratch" };

ostream& operator << (ostream& s, const tType& t) {
  s << BASETYPE_STR[t];
}

string tType::str() const {
  return BASETYPE_STR[_type];
}

ostream& operator << (ostream& s, const tValue& v) {
  switch(v.type()) {
    case tBOOLEAN:	s << ((boolean)v ? TRUE_STR : FALSE_STR); break;
    case tINTEGER:	s << (int)v; break;
    case tREAL:	   	s << (tReal)v; break;
    case tCOMPLEX:	s << (tComplex)v; break;
    case tSTRING:	s << (string)v; break;
    case tQUCONST:
    case tQUREG:
    case tQUVOID:
    case tQUSCR:	s << (quVar)v; break;
    default:		s << BASETYPE_STR[v.type()]; break;
  };
  return s;
}
 
void tValue::delpointer() {
  if(_type==tCOMPLEX) if(_val.pz) { qcl_delete(_val.pz); } else { cerr << "del _val.pz\n"; exit(1); }
  if(_type==tSTRING) if(_val.ps) { qcl_delete(_val.ps); } else { cerr << "del _val.ps\n"; exit(1); }
  if(_type.isQuExpr()) if(_val.pq) { qcl_delete(_val.pq); } else { cerr << "del _val.pq\n"; exit(1); }
}

string tValue::str() const {
  char s[80];
  switch(type()) {
    case tBOOLEAN:	return (_val.b ? TRUE_STR : FALSE_STR);
    case tINTEGER:
      sprintf(s,"%d",_val.i);
      return s;	
    case tREAL:
      sprintf(s,"%f",_val.r);
      return s;	
    case tCOMPLEX:
      sprintf(s,"(%f,%f)",_val.pz->real (),_val.pz->imag ());
      return s;	
    case tSTRING:	return (string)*(_val.ps);
    case tQUCONST:
    case tQUREG:
    case tQUVOID:
    case tQUSCR:
      {
        bitvec b(_val.pq->basebits()),c;
	string ostr="|";
	int i,j;
    	for(i=_val.pq->basebits()-1;i>=0;i--) {
    	  b.setbit(i,1);
    	  c=_val.pq->map(b);
    	  if(zero(c)) {
    	    ostr+=".";
    	  } else {
    	    for(j=0;j<_val.pq->mapbits();j++) {
    	      if(c[j]) { ostr+=(char)(j<10 ? '0'+j : 'a'+j-10); break; }
    	    }
    	  }
    	  b.setbit(i,0);
    	}
        return ostr+">";
      }
      //sprintf(s,"[%d / %d qubits]",_val.pq->mapbits(),_val.pq->basebits());
    default:		return BASETYPE_STR[type()];;
  };
  return "";
}

